import 'package:moor/moor.dart';
import 'package:unify_api/unify.dart' as api;

import '../database.dart';
import '../tables/videos.dart';

part 'video.g.dart';

@UseDao(tables: [
  VideoRecords
], queries: {
  'videosById':
      'SELECT * FROM videos WHERE user_id = :userId AND video_id = :videoId LIMIT 1',
  'videosByUser':
      'SELECT * FROM videos WHERE user_id = :userId ORDER BY views ASC',
})
class VideoDao extends DatabaseAccessor<DatabaseService> with _$VideoDaoMixin {
  VideoDao(DatabaseService db) : super(db);

  Future batchUpdate(String userId, List<api.Video> videos) async {
    final _items = <VideoRecordsCompanion>[];
    final _updated = DateTime.now();
    for (var video in videos) {
      final _item = VideoRecordsCompanion(
          videoId: Value(video.id),
          userId: Value(userId),
          name: addField<String>(video?.name),
          description: addField<String>(video?.description),
          thumbnail: addField<String>(video?.thumbnail),
          video: addField<String>(video?.video),
          download: addField<String>(video?.download),
          width: addField<int>(video?.width),
          height: addField<int>(video?.height),
          duration: addField<int>(video?.duration?.inSeconds),
          views: addField<int>(video?.views),
          plays: addField<int>(video?.plays),
          dateCreated: addField<DateTime>(video?.dateCreated),
          dateModified: addField<DateTime>(video?.dateModified),
          lastUpdated: Value(DateTime.now()));
      _items.add(_item);
    }
    await into(videoRecords).insertAll(_items, orReplace: true);
    await cleanVideos(userId, _updated);
  }

  Future<int> getTotalViews(String userId) async {
    int total = 0;
    final _videos = await videosByUser(userId);

    if (_videos != null) {
      for (var video in _videos) {
        total += video.views;
      }
    }
    return total;
  }

  Future<List<Video>> searchVideos(String userId, String query) {
    return (select(videoRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) {
            return or(l.videoId.like('%$query%'),
                or(l.name.like('%$query%'), l.description.like('%$query%')));
          })
          ..orderBy([
            (u) => OrderingTerm(expression: u.name, mode: OrderingMode.asc),
            (u) => OrderingTerm(expression: u.views, mode: OrderingMode.asc),
            (u) => OrderingTerm(expression: u.plays, mode: OrderingMode.asc),
          ]))
        .get();
  }

  Future<Video> getVideos(String userId, String videoId) async {
    final _videos = await videosById(userId, videoId);
    if (_videos != null && _videos.isNotEmpty) {
      return _videos.first;
    }
    return null;
  }

  Future<Video> getLastVideo(String userId) async {
    final _item = await (select(videoRecords)
          ..where((l) => l.userId.equals(userId))
          ..orderBy([
            (u) =>
                OrderingTerm(expression: u.lastUpdated, mode: OrderingMode.desc)
          ]))
        .get();
    if (_item != null && _item.isNotEmpty) {
      return _item.first;
    }
    return null;
  }

  Future resetVideosForUser(String userId) {
    return (delete(videoRecords)..where((c) => c.userId.equals(userId))).go();
  }

  Future cleanVideos(String userId, DateTime lastUpdated) {
    return (delete(videoRecords)
          ..where((l) => l.userId.equals(userId))
          ..where((l) => l.lastUpdated.isBiggerThanValue(lastUpdated)))
        .go();
  }
}
