import 'package:bloc/bloc.dart';
import 'package:example_db/data/blocs/settings/bloc.dart';
import 'package:example_db/data/blocs/settings/settings_bloc.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:get_it/get_it.dart';
import 'package:hydrated_bloc/hydrated_bloc.dart';

import 'data/blocs/alerts/bloc.dart';
import 'data/blocs/bloc.dart';
import 'data/blocs/calc/calculation_bloc.dart';
import 'data/blocs/videos/videos_bloc.dart';
import 'data/services/database/database.dart';
import 'data/services/database/database/shared.dart';
import 'generated/i18n.dart';
import 'plugins/desktop/desktop.dart';
import 'ui/auth/screen.dart';
import 'ui/router.dart';
import 'ui/search/screen.dart';
import 'ui/theme.dart';

GetIt getIt = GetIt.instance;

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  BlocSupervisor.delegate = SimpleBlocDelegate();
  getIt.registerSingleton<DatabaseService>(constructDb());
  setTargetPlatformForDesktop();
  // initializeDateFormatting().then((_) => runApp(MyApp()));
  BlocSupervisor.delegate = await HydratedBlocDelegate.build();
  runApp(MyApp());
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  final _settings = SettingsBloc();
  final _nav = GlobalKey<NavigatorState>();
  final _scaffoldKey = GlobalKey<ScaffoldState>();

  @override
  void initState() {
    _settings.dispatch(CheckSettings());
    super.initState();
  }

  @override
  void dispose() {
    _settings.dispose();
    super.dispose();
  }

  bool _fresh = true;
  @override
  Widget build(BuildContext context) {
    final i18n = I18n.delegate;
    return MultiBlocProvider(
      providers: [
        BlocProvider<NavBloc>(builder: (_) => NavBloc(_nav)),
        BlocProvider<ScaffoldBloc>(builder: (_) => ScaffoldBloc(_scaffoldKey)),
        BlocProvider<AuthBloc>(builder: (_) => AuthBloc()),
        BlocProvider<SettingsBloc>(builder: (_) => _settings),
        BlocProvider<LoansBloc>(builder: (_) => LoansBloc()),
        BlocProvider<SearchBloc>(builder: (_) => SearchBloc()),
        BlocProvider<ContactsBloc>(builder: (_) => ContactsBloc()),
        BlocProvider<AlertsBloc>(builder: (_) => AlertsBloc()),
        BlocProvider<LeadsBloc>(builder: (_) => LeadsBloc()),
        BlocProvider<CalendarBloc>(builder: (_) => CalendarBloc()),
        BlocProvider<VideosBloc>(builder: (_) => VideosBloc()),
        BlocProvider<CalculationBloc>(builder: (_) => CalculationBloc()),
      ],
      child: BlocListener<SettingsBloc, SettingsState>(
        listener: (context, settingsState) {
          if (_fresh && settingsState is SettingsReady) {
            if (settingsState.settings.autoLogin) {
              BlocProvider.of<AuthBloc>(context).dispatch(CheckUser());
              _fresh = false;
            }
          }
        },
        child: BlocBuilder<SettingsBloc, SettingsState>(
          builder: (context, settingsState) => MaterialApp(
            navigatorKey: _nav,
            debugShowCheckedModeBanner: false,
            onGenerateTitle: (context) => I18n.of(context).title,
            localizationsDelegates: [
              i18n,
              GlobalMaterialLocalizations.delegate,
              GlobalWidgetsLocalizations.delegate,
            ],
            supportedLocales: i18n.supportedLocales,
            localeResolutionCallback: i18n.resolution(
              fallback: Locale('en', 'US'),
            ),
            theme: settingsState is SettingsReady &&
                    settingsState.settings.darkMode
                ? AppColors.dark
                : AppColors.light,
            darkTheme: settingsState is SettingsReady &&
                    settingsState.settings.useSystemDark
                ? AppColors.dark
                : null,
            initialRoute: Router.initialRoute,
            routes: Router.routes(context),
            onUnknownRoute: (settings) => Router.onUnknownRoute(
              context,
              settings: settings,
            ),
            onGenerateRoute: (settings) => Router.onUnknownRoute(
              context,
              settings: settings,
            ),
          ),
        ),
      ),
    );
  }

  static Route onGenerateRoute(
    BuildContext context, {
    @required RouteSettings settings,
  }) {
    final path = settings.name;
    final Map<String, dynamic> args = settings?.arguments;
    switch (path) {
      case SearchScreen.routeName:
        if (args != null) {
          final int _index = args['type'];
          return _buildRoute(
            settings,
            SearchScreen(
              scopes: _index == null ? null : [SearchResultType.values[_index]],
            ),
          );
        }
        return _buildRoute(settings, SearchScreen());
      default:
        return Router.onUnknownRoute(context, settings: settings);
    }
  }

  static MaterialPageRoute _buildRoute(RouteSettings settings, Widget builder) {
    return MaterialPageRoute(
      settings: settings,
      builder: (context) => builder,
    );
  }
}
