import 'package:flutter/material.dart';
import 'package:unify_api/unify.dart';

import '../../../../data/utils/debouncer.dart';

class CampaignSelect extends StatefulWidget {
  const CampaignSelect({
    Key key,
    @required this.onSelect,
    @required this.client,
    @required this.material,
  }) : super(key: key);

  final UnifyClient client;
  final MarketingMaterial material;
  final ValueChanged<MarketingMaterial> onSelect;

  @override
  _CampaignSelectState createState() => _CampaignSelectState();
}

class _CampaignSelectState extends State<CampaignSelect> {
  final _controller = TextEditingController();
  final _debouncer = Debouncer();
  MarketingCategory _marketingCategory;
  MarketingSubCategory _marketingSubCategory;
  MarketingMaterial _material;
  String _query = "";
  List<MarketingSection> _sections = [];
  MarketingSection _selectedSection;
  bool _showMaterial = false;

  @override
  void initState() {
    _material = widget?.material;
    widget.client.listMarketingSection().then((items) {
      if (mounted)
        setState(() {
          _sections = items;
        });
    });
    super.initState();
  }

  void _clearSearch() {
    if (mounted) {
      setState(() {
        _controller.clear();
        _query = '';
      });
    }
  }

  double kGridKeight(BuildContext context) =>
      MediaQuery.of(context).size.height * 0.5;

  Widget _buildMaterialCategories(BoxConstraints dimens) {
    return FutureBuilder<List<MarketingMaterial>>(
        future: widget.client.listMarketingMaterial(
          type: _selectedSection?.name,
          category: _marketingCategory?.name,
          subCategory: _marketingSubCategory?.name,
          query: _query,
        ),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            if (snapshot.data.isEmpty) {
              return Center(
                child: Text('No Campaigns Found'),
              );
            }
            return Container(
              height: kGridKeight(context),
              child: GridView.builder(
                gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                  crossAxisCount: (dimens.maxWidth / 200).ceil(),
                ),
                // shrinkWrap: true,
                itemCount: snapshot.data.length,
                itemBuilder: (context, index) {
                  final item = snapshot.data[index];
                  return Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: InkWell(
                      onTap: () {
                        if (mounted)
                          setState(() {
                            _material = item;
                          });
                        widget.onSelect(item);
                      },
                      child: Banner(
                        location: BannerLocation.topEnd,
                        message: item.price == 0
                            ? 'Free'
                            : '\$${item.price.toStringAsFixed(2)}',
                        child: Stack(
                          children: <Widget>[
                            Positioned.fill(
                              child: MarketingCard(
                                image: item.imageUrl,
                                name: item.name,
                                showLabel: true,
                              ),
                            ),
                            if (_material != null &&
                                _material.id == item.id) ...[
                              Container(
                                color: Colors.blueAccent.withOpacity(0.2),
                              ),
                            ],
                          ],
                        ),
                      ),
                    ),
                  );
                },
              ),
            );
          }
          return Container(
            height: kGridKeight(context),
            child: Center(
              child: CircularProgressIndicator(),
            ),
          );
        });
  }

  Widget _buildSubCategories(BoxConstraints dimens) {
    return Container(
      height: kGridKeight(context),
      child: GridView.builder(
        gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
          crossAxisCount: (dimens.maxWidth / 200).ceil(),
        ),
        // shrinkWrap: true,
        itemCount: _marketingCategory.items.length,
        itemBuilder: (context, index) {
          final item = _marketingCategory.items[index];
          return Padding(
            padding: const EdgeInsets.all(8.0),
            child: InkWell(
              onTap: () {
                if (mounted)
                  setState(() {
                    _marketingSubCategory = item;
                  });
              },
              child: MarketingCard(
                image: item.imageUrl,
                name: item.name,
              ),
            ),
          );
        },
      ),
    );
  }

  Widget _buildCategories(BoxConstraints dimens) {
    return Container(
      height: kGridKeight(context),
      child: GridView.builder(
        gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
          crossAxisCount: (dimens.maxWidth / 200).ceil(),
        ),
        // shrinkWrap: true,
        itemCount: _selectedSection.items.length,
        itemBuilder: (context, index) {
          final item = _selectedSection.items[index];
          return Padding(
            padding: const EdgeInsets.all(8.0),
            child: InkWell(
              onTap: () {
                if (mounted)
                  setState(() {
                    _marketingCategory = item;
                    if (_marketingCategory?.items == null ||
                        _marketingCategory.items.isEmpty) {
                      _showMaterial = true;
                    }
                  });
              },
              child: MarketingCard(
                image: item.imageUrl,
                name: item.name,
              ),
            ),
          );
        },
      ),
    );
  }

  Widget _buildSections(BoxConstraints dimens) {
    return Container(
      height: kGridKeight(context),
      child: GridView.builder(
        gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
          crossAxisCount: (dimens.maxWidth / 200).ceil(),
        ),
        // shrinkWrap: true,
        itemCount: _sections.length,
        itemBuilder: (context, index) {
          final item = _sections[index];
          return Padding(
            padding: const EdgeInsets.all(8.0),
            child: InkWell(
              onTap: () {
                if (mounted)
                  setState(() {
                    _selectedSection = item;
                  });
              },
              child: MarketingCard(
                image: item.imageUrl,
                name: item.name,
              ),
            ),
          );
        },
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (context, dimens) => Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Wrap(
            children: <Widget>[
              if (_selectedSection != null)
                Padding(
                  padding: const EdgeInsets.all(2.0),
                  child: Chip(
                    label: Text(_selectedSection.name),
                    onDeleted: () {
                      if (mounted)
                        setState(() {
                          _selectedSection = null;
                          _marketingCategory = null;
                          _marketingSubCategory = null;
                          _showMaterial = false;
                        });
                    },
                  ),
                ),
              if (_marketingCategory != null)
                Padding(
                  padding: const EdgeInsets.all(2.0),
                  child: Chip(
                    label: Text(_marketingCategory.name),
                    onDeleted: () {
                      if (mounted)
                        setState(() {
                          _marketingCategory = null;
                          _marketingSubCategory = null;
                          _showMaterial = false;
                        });
                    },
                  ),
                ),
              if (_marketingSubCategory != null)
                Padding(
                  padding: const EdgeInsets.all(2.0),
                  child: Chip(
                    onDeleted: () {
                      if (mounted)
                        setState(() {
                          _marketingSubCategory = null;
                          _showMaterial = false;
                        });
                    },
                    label: Text(_marketingSubCategory.name),
                  ),
                ),
              if (_showMaterial)
                Padding(
                  padding: const EdgeInsets.all(2.0),
                  child: Chip(
                    onDeleted: () {
                      if (mounted)
                        setState(() {
                          _showMaterial = false;
                          _clearSearch();
                          if (_marketingCategory?.items == null ||
                              _marketingCategory.items.isEmpty) {
                            _marketingCategory = null;
                          }
                        });
                    },
                    label: Text('Material'),
                  ),
                ),
            ],
          ),
          Container(
            padding: EdgeInsets.all(4.0),
            child: TextField(
              decoration: InputDecoration(
                  hintText: 'Search..',
                  border: InputBorder.none,
                  filled: true,
                  suffixIcon: IconButton(
                    icon: Icon(Icons.clear),
                    onPressed: _clearSearch,
                  )),
              controller: _controller,
              onChanged: (val) {
                _debouncer.run(() {
                  if (mounted)
                    setState(() {
                      _query = val;
                      _showMaterial = true;
                    });
                });
              },
            ),
          ),
          Padding(
            padding: const EdgeInsets.only(top: 8.0),
            child: Builder(
              builder: (_) {
                if (_showMaterial || _marketingSubCategory != null) {
                  return _buildMaterialCategories(dimens);
                }
                if (_marketingCategory != null) {
                  return _buildSubCategories(dimens);
                }
                if (_selectedSection != null) {
                  return _buildCategories(dimens);
                }
                return _buildSections(dimens);
              },
            ),
          ),
        ],
      ),
    );
  }
}

class MarketingCard extends StatelessWidget {
  const MarketingCard({
    Key key,
    @required this.image,
    @required this.name,
    this.showLabel = false,
  }) : super(key: key);

  final String image, name;
  final bool showLabel;

  @override
  Widget build(BuildContext context) {
    return Column(
      children: <Widget>[
        Expanded(
          child: Image.network(
            image,
            fit: BoxFit.fill,
          ),
        ),
        if (showLabel)
          Container(
            height: 50,
            color: Theme.of(context).accentColor,
            child: Center(
              child: Row(
                children: <Widget>[
                  Expanded(
                    child: Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Text(
                        name,
                        textAlign: TextAlign.center,
                        style: TextStyle(
                            fontSize: 16.0,
                            fontWeight: FontWeight.w500,
                            color: Colors.white),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
      ],
    );
  }
}
