import 'dart:io';

import 'package:example_db/data/utils/navigate.dart';
import 'package:example_db/ui/video/common/upload_video.dart';
import 'package:fab_menu/fab_menu.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:get_it/get_it.dart';
import 'package:image_picker/image_picker.dart';
import 'package:unicorndial/unicorndial.dart';
import 'package:unify_api/unify.dart' as api;

import '../../../data/blocs/auth/bloc.dart';
import '../../../data/blocs/scaffold/scaffold_bloc.dart';
import '../../../data/blocs/videos/bloc.dart';
import '../../../data/blocs/videos/videos_bloc.dart';
import '../../../data/classes/enum_list.dart';
import '../../../data/services/database/database.dart';
import '../../../plugins/video_player/video_player.dart';
import 'tile.dart';

class VideoListScreen extends StatefulWidget {
  @override
  _VideoListScreenState createState() => _VideoListScreenState();

  static refresh(BuildContext context) {
    final videos = BlocProvider.of<VideosBloc>(context);
    final _user = AuthBloc.currentInfo(context);
    final scaffold = BlocProvider.of<ScaffoldBloc>(context);
    return videos.dispatch(
        RefreshVideos(user: _user, scaffold: scaffold, context: context));
  }
}

class _VideoListScreenState extends State<VideoListScreen> {
  File _image;
  int _currenIndex = 0;
  List<UnicornButton> childButtons = [];

  @override
  void initState() {
    super.initState();
    childButtons.add(UnicornButton(
      hasLabel: true,
      labelText: 'Add Video',
      currentButton: FloatingActionButton(
        heroTag: 'video_select',
        mini: true,
        //backgroundColor: Theme.of(context).accentColor,
        child: Icon(Icons.add_photo_alternate),
        onPressed: () {
          selectVid();
          if (_image != null){
            navigate(context, VideoUpload(video: _image));
          }
        },
      ),
    ));
    childButtons.add(UnicornButton(
      hasLabel: true,
      labelText: 'Record Video',
      currentButton: FloatingActionButton(
        heroTag: 'record_video',
        mini: true,
        //backgroundColor: Theme.of(context).accentColor,
        child: Icon(FontAwesomeIcons.video),
        onPressed: () {
          recordVid();
          if (_image != null){
            navigate(context, VideoUpload(video: _image,));
          }
        },
      ),
    ));
  }

  @override
  Widget build(BuildContext context) {
    final videos = BlocProvider.of<VideosBloc>(context);
    final _user = AuthBloc.currentUser(context);
    final bool test = false;
    final _folders = [
      'My Videos',
      'Shared',
      'Archived',
    ];
    return DefaultTabController(
      initialIndex: _currenIndex,
      length: _folders.length,
      child: Scaffold(
        appBar: PreferredSize(
          preferredSize: Size.fromHeight(60),
          child: Center(
            child: TabBar(
              onTap: (val) => _currenIndex = val,
              isScrollable: true,
              tabs: <Widget>[
                for (var folder in _folders) ...[Tab(text: folder)],
              ],
            ),
          ),
        ),
        floatingActionButton: (_user.videoEnabled) ? UnicornDialer(
          parentHeroTag: 'new_video',
          parentButton: Icon(Icons.add),
          childButtons: childButtons,
          orientation: UnicornOrientation.VERTICAL,
        ) : null,
        body: TabBarView(
          children: <Widget>[
            MyVideosList(videos: videos, user: _user),
            MySharedList(),
            MyArchivedList(),
          ],
        ),
      ),
    );
  }
  Future recordVid() async {
    final image = await ImagePicker.pickVideo(source: ImageSource.camera);
    setState(() {
      _image = image;
    });
  }
  Future selectVid() async {
    final image = await ImagePicker.pickVideo(source: ImageSource.gallery);
    setState(() {
      _image = image;
    });
  }
}


class MyVideosList extends StatelessWidget {
  const MyVideosList({
    Key key,
    @required this.videos,
    @required User user,
  })  : _user = user,
        super(key: key);

  final VideosBloc videos;
  final User _user;

  @override
  Widget build(BuildContext context) {
    return Container(
      child: StreamBuilder<VideosState>(
          stream: videos.state,
          builder: (context, snapshot) {
            final _state = snapshot.data;
            return Column(
              children: <Widget>[
                if (_state is VideosLoadingState) LinearProgressIndicator(),
                Expanded(
                  child: Container(
                      child: StreamBuilder<List<Video>>(
                          stream: GetIt.instance<DatabaseService>()
                              .videoDao
                              .watchVideosByUser(_user.userID),
                          builder: (context, snapshot) {
                            if (snapshot.hasData) {
                              final _videos = snapshot.data;
                              if (_videos.isEmpty) {
                                if (_state is VideosLoadingState) {
                                  return Column(
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    crossAxisAlignment:
                                        CrossAxisAlignment.center,
                                    children: [
                                      Text('No Videos Found'),
                                      SizedBox(height: 20.0),
                                      CircularProgressIndicator(),
                                    ],
                                  );
                                }
                                return Center(child: Text('No Videos Found'));
                              }
                              return Scrollbar(
                                child: RefreshIndicator(
                                  child: ListView.separated(
                                      separatorBuilder: (context, index) =>
                                          Divider(height: 1),
                                      itemCount: _videos.length,
                                      itemBuilder: (context, index) {
                                        final _video = _videos[index];
                                        return StreamBuilder<List<VideoFolder>>(
                                          stream:
                                              GetIt.instance<DatabaseService>()
                                                  .videoFolderDao
                                                  .watchVideosById(_user.userID,
                                                      _video.videoId),
                                          builder: (context, vid) {
                                            if (vid.hasData &&
                                                vid.data.isEmpty) {
                                              return Container(
                                                padding:
                                                    const EdgeInsets.all(7),
                                                child: VideoRowTile(
                                                  video: _video,
                                                  type: ViewType.detailed,
                                                ),
                                              );
                                            }
                                            return Container();
                                          },
                                        );
                                      }),
                                  onRefresh: () async {
                                    VideoListScreen.refresh(context);
                                    await Future.delayed(Duration(seconds: 3));
                                  },
                                ),
                              );
                            }
                            return Center(child: CircularProgressIndicator());
                          })),
                )
              ],
            );
          }),
    );
  }
  
}

class MySharedList extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    final user = AuthBloc.currentInfo(context);
    return Container(
      child: FutureBuilder<List<api.Video>>(
        future: user.client.listVideosShared(),
        builder: (context, list) {
          if (!list.hasData || list.data == null) {
            return Center(child: CircularProgressIndicator());
          } else {
            if (list.data.isEmpty) {
              return Center(child: Text('No Shared Videos'));
            }
            return SingleChildScrollView(
              child: Column(
                children: <Widget>[
                  for (var vid in list.data) ...[
                    ExpansionTile(
                      title: ListTile(
                        leading: Icon(FontAwesomeIcons.video),
                        title: Text('${vid.name}'),
                        subtitle: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Text(
                              vid.description,
                              maxLines: 1,
                              softWrap: true,
                              overflow: TextOverflow.ellipsis,
                            ),
                            Text('${vid.views} views')
                          ],
                        ),
                      ),
                      children: <Widget>[
                        Container(
                          child: VideoPlayerView(
                            src: vid.download,
                            width: vid.width,
                            height: vid.height,
                          ),
                        )
                      ],
                    ),
                  ]
                ],
              ),
            );
          }
        },
      ),
    );
  }
}

class MyArchivedList extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    final userInfo = AuthBloc.currentInfo(context);
    return Container(
      child: StreamBuilder<List<VideoFolder>>(
        stream: GetIt.instance<DatabaseService>()
            .videoFolderDao
            .watchArchivedVideos(userInfo.id),
        builder: (context, list) {
          if (!list.hasData || list.data == null) {
            return Center(child: CircularProgressIndicator());
          } else {
            if (list.data.isEmpty) {
              return Center(child: Text('No Archived Videos'));
            }
            return SingleChildScrollView(
              child: Column(
                children: <Widget>[
                  for (var vid in list.data) ...[
                    FutureBuilder<Video>(
                      future: GetIt.instance<DatabaseService>()
                          .videoDao
                          .getVideos(userInfo.id, vid.videoId),
                      builder: (context, video) {
                        if (video.data != null && video.hasData) {
                          return Container(
                            child: VideoRowTile(
                              archived: true,
                              video: video.data,
                              type: ViewType.detailed,
                            ),
                          );
                        }
                        return Container();
                      },
                    )
                  ]
                ],
              ),
            );
          }
        },
      ),
    );
  }
}
