part of unify;

class Lead implements UnifyObject, QueryObject, SortableObject<Lead> {
  Lead(this.client, this.json);

  @override
  final UnifyClient client;

  @override
  final Map<String, dynamic> json;

  String get id => json["Lead_ID"];
  String get contactId => json["Contact_ID"];
  String get fullName => "$firstName $lastName".trim();
  String get displayName => "$lastName, $firstName";
  String get firstName => json["First_Name"];
  String get lastName => json["Last_Name"];
  PhoneNumber get cellPhone => PhoneNumber(client, json["Cell_Phone"]);
  PhoneNumber get homePhone => PhoneNumber(client, json["Home_Phone"]);
  PhoneNumber get officePhone => PhoneNumber(client, json["Office_Phone"]);
  String get email => json["Email_Address"];
  String get lastActivity => json["Last_Activity"];
  DateTime get dateCreated => DateTime.tryParse(json["Date_Created"] as String);
  DateTime get dateModified =>
      DateTime.tryParse(json["Date_Modified"] as String);

  List<PhoneNumber> get numbers {
    List<PhoneNumber> _items = [];
    if (cellPhone != null && cellPhone.raw().isNotEmpty) _items.add(cellPhone);
    if (homePhone != null && homePhone.raw().isNotEmpty) _items.add(homePhone);
    if (officePhone != null && officePhone.raw().isNotEmpty)
      _items.add(officePhone);
    return _items ?? [];
  }

  // -- Actions --
  Future<LeadDetails> getDetails() {
    return client.getLeadDetails(id);
  }

  Future<List<LeadNote>> listNotes() {
    return client.listLeadNotes(id);
  }

  Future<List<Tracker>> listTracker() {
    return client.listLeadTracker(id);
  }

  Future<List<Task>> listTasks({DateTime date}) {
    return client.listTasksForLead(id, date: date);
  }

  Future<List<String>> listResponses() {
    return client.listLeadLogResponses();
  }

  Future addNote(String text) {
    return client.addLeadNote(id, text);
  }

  Future addLogResponse({@required String action, String comment}) {
    return client.addLeadLogResponse(id, action: action, comment: comment);
  }

  Future delete() {
    return client.deleteLead(id);
  }

  /// type: `0 (Task)`, `1 (Appointment)`, `2 (Scheduled Call)`
  ///
  /// alertType: `Email`, `TextMessage`, `Email, TextMessage`, ""
  ///
  /// alertOffset: int for `minutes`
  Future addFollowUp({
    @required String title,
    DateTime date,
    int type = 0,
    String description = "",
    bool dismissOnChange = true,
    String alertType,
    int alertOffset = 0,
    String userNumber,
    String prospectNumber,
  }) {
    return client.addLeadFollowUp(
      id,
      title: title,
      date: date,
      type: type,
      description: description,
      dismissOnChange: dismissOnChange,
      alertType: alertType,
      alertOffset: alertOffset,
      userNumber: userNumber,
      prospectNumber: prospectNumber,
    );
  }

  Future edit({
    @required String firstName,
    @required String lastName,
    PhoneNumber cellPhone,
    PhoneNumber homePhone,
    PhoneNumber officePhone,
    String email,
  }) {
    return client.editLead(
      id,
      firstName: firstName,
      lastName: lastName,
      cellPhone: cellPhone,
      homePhone: homePhone,
      officePhone: officePhone,
      email: email,
    );
  }

  @override
  bool matchesSearch(String search) {
    if (search == null || search.isEmpty) {
      return true;
    }

    search = search.toLowerCase();

    if (firstName.toLowerCase().contains(search)) {
      return true;
    }
    if (lastName.toLowerCase().contains(search)) {
      return true;
    }
    if (cellPhone.raw().toLowerCase().contains(search)) {
      return true;
    }
    if (homePhone.raw().toLowerCase().contains(search)) {
      return true;
    }
    if (officePhone.raw().toLowerCase().contains(search)) {
      return true;
    }
    if (email.toLowerCase().contains(search)) {
      return true;
    }
    if (dateCreated.toString().toLowerCase().contains(search)) {
      return true;
    }
    if (dateModified.toString().toLowerCase().contains(search)) {
      return true;
    }
    if (lastActivity.toLowerCase().contains(search)) {
      return true;
    }
    return false;
  }

  @override
  List<String> sortFields = [
    "first_name",
    "last_name",
    "cell_phone",
    "office_phone",
    "home_phone",
    "date_created",
    "date_modified",
    "email",
    "last_activity",
  ];

  @override
  int compareTo(Lead object, String sortField, bool sortAscending) {
    int response = 0;
    Lead objectA = sortAscending ? this : object;
    Lead objectB = sortAscending ? object : this;

    switch (sortField) {
      case "first_name":
        response = objectA.firstName.compareTo(objectB.firstName);
        break;
      case "last_name":
        response = objectA.lastName.compareTo(objectB.lastName);
        break;
      case "cell_phone":
        response = objectA.cellPhone.raw().compareTo(objectB.cellPhone.raw());
        break;
      case "office_phone":
        response =
            objectA.officePhone.raw().compareTo(objectB.officePhone.raw());
        break;
      case "home_phone":
        response = objectA.homePhone.raw().compareTo(objectB.homePhone.raw());
        break;
      case "date_created":
        response = objectA.dateCreated.compareTo(objectB.dateCreated);
        break;
      case "date_modified":
        response = objectA.dateModified.compareTo(objectB.dateModified);
        break;
      case "email":
        response = objectA.email.compareTo(objectB.email);
        break;
      case "last_activity":
        response = objectA.lastActivity.compareTo(objectB.lastActivity);
        break;
    }

    if (response == 0) {
      return objectA.firstName.compareTo(objectB.firstName);
    } else {
      return response;
    }
  }
}
